<!-- Template for network attack surface -->
<script id="service_groups.database.summaries.external_attack_surface.details.template" type="text/x-handlebars-template">
  {{> attack_surface service = 'Database'}}
</script>

