<!-- Filters -->
<script id="filters.list.template" type="text/x-handlebars-template">
  {{#each items}}
    {{#each filters}}   
      <li id="services.{{display_path}}.filters"><a class="dropdown-item" href="#services.{{@../key}}.filters.{{@key}}.items">{{description}}</a></li>
    {{/each}}
  {{/each}}
</script>
