@import url("./modal.css");

/* Bootstrap overrides */

.btn-success,
.btn-success:focus
 {
    color: #fff;
    background-color: #23af44;
    border-color: #23af44;
}

.btn-success:hover,
.btn-success:active
 {
    color: #fff;
    background-color: #1f9b3c;
    border-color: #1f9b3c;
}

.btn-warning:focus {
    color: #fff;
    background-color: #F39C12;
    border-color: #F39C12;
}

.btn-danger:focus {
    color: #fff;
    background-color: #E74C3C;
    border-color: #E74C3C;
}

.btn-info.disabled, .btn-info:disabled {
    color: rgb(255, 255, 255);
    background-color: rgb(180, 180, 180);
    border-color: rgb(180, 180, 180);
}

/* Scout Suite */


.item-margin {
    margin-left: 25px;
}
.no-margin {
    margin-left: -25px !important;
}

.finding-title {
    background: #f5f5f5;
}

.finding-badge {
    margin-bottom: 1px;
    margin-right: 8px;
    display: inline-block;
    border-radius: 100%;
}

.finding-badge-good:before { 
    content: '\f058';
    color: #23af44; 
}

.finding-badge-warning:before { 
    content: '\f06a';
    color: #fdbe00; 
}

.finding-badge-danger:before { 
    content: '\f06a';
    color:#ee293d; 
}
.finding-badge-unknown:before {  
    content: '\f111';
    color: rgb(225, 225, 225);
}

.finding-warning { color: #fdbe00; }
.finding-danger { color:#ee293d; }

ul.no-bullet {
    list-style-type: none;
}
.list-sub-element {
    border: 0 !important;
    padding-left: 25px;
}

.active-dropdown {
    color: #428bca;
}

.overlay-bg {
    position: fixed;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
    background: black;
    z-index: 1337;
    opacity: 0.5;
    display: none;
}
.overlay {
    position: fixed;
    top: 40%;
    left: 40%;
    max-height: 80%;
    background-color: white;
    z-index: 7331;
    overflow: auto;
    margin-left: -200px;
    margin-top: -200px;
    display: none;
    border-radius: 5px;
}

.plain-link a:link,
.plain-link a:visited,
.plain-link a:active {
    color: black;
    text-decoration: none;
}

.plain-link a:hover {
    text-decoration: underline;
}

.disabled-link a:link,
.disabled-link a:active,
.disabled-link a:visited,
.disabled-link a:hover {
    color: rgb(110, 110, 110);
    pointer-events: none;
    cursor: default;
}
.active a{
    text-decoration: none;
    color: black;
}

.dropdown-submenu {
    position: relative;
}

.dropdown-submenu>.dropdown-menu {
    top: 0;
    left: 100%;
    margin-top: -6px;
    margin-left: -1px;
    -webkit-border-radius: 0 6px 6px 6px;
    -moz-border-radius: 0 6px 6px;
    border-radius: 0 6px 6px 6px;
}

.dropdown-submenu:hover>.dropdown-menu {
    display: block;
}

.dropdown-submenu>a:after {
    display: block;
    content: " ";
    float: right;
    width: 0;
    height: 0;
    border-color: transparent;
    border-style: solid;
    border-width: 5px 0 5px 5px;
    border-left-color: #ccc;
    margin-top: 5px;
}

.dropdown-submenu:hover>a:after {
    border-left-color: #fff;
}

.dropdown-submenu.pull-left {
    float: none;
}

.dropdown-submenu.pull-left>.dropdown-menu {
    left: -100%;
    margin-left: 10px;
    -webkit-border-radius: 6px 0 6px 6px;
    -moz-border-radius: 6px 0 6px 6px;
    border-radius: 6px 0 6px 6px;
}

.dropdown-item.disabled, .dropdown-item:disabled {
    color: #bbcdce;
}

.rationale-overlay {
    z-index: 2147483647;
    display: block;
    position: fixed;
    width: 20%;
    display: none;
}

.switch {
    position: relative;
    display: inline-block;
    width: 26px;
    height: 14px;
    line-height: 1em;
  }

.switch input { 
    opacity: 0;
    width: 0;
    height: 0;
}

.slider {
    position: absolute;
    cursor: pointer;
    top: 0;
    left: 0;
    right: 0;
    bottom: 0;
    background-color: #ccc;
}

.slider:before {
    position: absolute;
    content: "";
    height: 11px;
    width: 11px;
    left: 2px;
    bottom: 2px;
    background-color: white;
}

input:checked + .slider {
    background-color: #428bca;
}

input:focus + .slider {
    box-shadow: 0 0 1px 1px #428bca;
}

input:checked + .slider:before {
    -ms-transform: translateX(11px);
    transform: translateX(11px);
}

.slider.round {
    border-radius: 14px;
    top: 2px;
    bottom: -2px;
}

.slider.round:before {
    border-radius: 50%;
}

.sub-navbar {
    position: inherit;
    float: bottom;
    margin-top: 3rem;
    width: 100%;
    left: 0rem;
    right: 0rem;
    top: 0rem;
    padding-top: 2rem;
    padding-left: 8rem;
    align-content: stretch;
    z-index: -1;
}

.pr-0 {
    padding-right: 0rem;
}

.text-overflow {
    word-wrap: break-word;
}

.std-size {
    font-size: 18px;
}

.finding {
    position: relative;
}

.finding:before {
    color: black;
    content: "\f068"; /* Fontawesome '+' */
    left:5px;
    position:absolute;
    top:0;
}


.collapsed.finding:before {
    color: black;
    content: "\f067"; /* Fontawesome '-' */
    left:5px;
    position:absolute;
    top:0;
}

#last_run {
    width: 100%;
}

.list-group {
    margin-bottom: 1em;
}

#double-column-left .list-group {
    margin: 3px 0;
}

#double-column-left .active a,
#double-column-left .active .fa {
    color: rgba(24, 188, 155);
}

#double-column-left .active .fa:hover {
    color: rgba(24, 188, 155);
}

.finding-learn-more i {
    font-size: 0.8em;
}

.finding_items {
    font-size: 18px;
}

@media only screen and (max-width: 1200px) {
    #container-user {
      margin-top: 5rem !important;
    }
  }