<!-- Application Gateway partial -->
<script id="services.appgateway.app_gateways.partial" type="text/x-handlebars-template">
    <div id="resource-name" class="list-group-item active">
        <h4 class="list-group-item-heading">{{name}}</h4>
    </div>

    <div class="list-group-item">
        <h4 class="list-group-item-heading">Information</h4>
        <div class="list-group-item-text item-margin">Web Application Firewall: <span id="appgateway.app_gateways.{{@key}}.web_app_firewall_enabled">{{ convert_bool_to_enabled web_app_firewall_enabled }}</span></div>
    </div>

</script>

<script>
    Handlebars.registerPartial("services.appgateway.app_gateways", $("#services\\.appgateway\\.app_gateways\\.partial").html());
</script>

<!-- Single Application Gateway template -->
<script id="single_appgateway_app_gateways-template" type="text/x-handlebars-template">
    {{> modal-template template='services.appgateway.app_gateways' }}
</script>
<script>
    var single_appgateway_app_gateways_template = Handlebars.compile($("#single_appgateway_app_gateways-template").html());
</script>