
<!-- Single Redshift cluster template -->
<script id="single_redshift_cluster-template" type="text/x-handlebars-template">
    {{> modal-template template='redshift_cluster'}}
</script>
<script>
    var single_redshift_cluster_template = Handlebars.compile($("#single_redshift_cluster-template").html());
</script>

