    <!-- Generic link to other resources -->
    <script id="resource_link.partial" type="text/x-handlebars-template">
        <a href="javascript:showObject('{{resource_path}}')">
            {{get_value_at resource_path 'name'}}
        </a>
    </script>
    <script>
        Handlebars.registerPartial("resource_link", $("#resource_link\\.partial").html());
    </script>