
    <!-- elb ELB partial -->
    <script id="services.elb.regions.vpcs.elbs.linked_resources.partial" type="text/x-handlebars-template">
      {{#if resources.length}}
        <div class="accordion">
          <li>
            <h5 class="list-group-item-heading accordion-heading">{{make_title resource_type}}:
              {{> count_badge count=resources.length target=(concat '#services.elb.regions' region 'vpcs' vpc 'elbs' @key resource_type)}}
            </h5>
          </li>
          <div id="services.elb.regions.{{region}}.vpcs.{{vpc}}.elbs.{{@key}}.{{resource_type}}" class="accordion-body collapse">
            <div class="accordion-inner">
              <ul>
              {{#each resources}}
                <li class="list-group-item-text"><a href="javascript:showObject('services.elb.regions.{{../region}}.vpcs.{{../vpc}}.{{../resource_type}}.{{this}}')">{{this}}</a></li>
              {{/each}}
              </ul>
            </div>
          </div>
        </div>
      {{else}}
        <h5 class="list-group-item-heading item-margin accordion-heading">{{make_title resource_type}}:<span class="badge float-right">{{resources.length}}</span></h5>
      {{/if}}
    </script>
    <script>
        Handlebars.registerPartial("services.elb.regions.vpcs.elbs.linked_resources", $("#services\\.elb\\.regions\\.vpcs\\.elbs\\.linked_resources\\.partial").html());
    </script>
