
  <!-- IAM credential report partial -->
  <script id="services.iam.credential_reports.partial" type="text/x-handlebars-template">
    <div id="resource-name" class="list-group-item active">
      <h4 class="list-group-item-heading">{{name}}</h4>
    </div>
    <div class="list-group-item">
        <h4 class="list-group-item-heading">Credentials Report</h4>
        <div class="list-group-item-text item-margin">Creation date: {{ format_date (get_value_at 'services' 'iam' 'credential_reports' id 'user_creation_time')}}</div>
        <div class="list-group-item-text item-margin">Last used date:  <span id="iam.credential_reports.{{id}}.inactive_user">{{ format_date (get_value_at 'services' 'iam' 'credential_reports' id 'last_used')}}</span></div>
        <div class="list-group-item-text item-margin">Password last used: <span id="iam.credential_reports.{{id}}.password_last_used">{{ format_date (get_value_at 'services' 'iam' 'credential_reports' id 'password_last_used')}}</span></div>
        <div class="list-group-item-text item-margin">MFA enabled: <span id="iam.credential_reports.{{id}}.mfa_active">{{get_value_at 'services' 'iam' 'credential_reports' id 'mfa_active'}}</span></div>
        <div class="list-group-item-text item-margin">Access key 1 active: <span id="iam.credential_reports.{{id}}.access_key_1_active">{{get_value_at 'services' 'iam' 'credential_reports' id 'access_key_1_active'}}</span></div>
        <div class="list-group-item-text item-margin">Access key 2 active: <span id="iam.credential_reports.{{id}}.access_key_2_active">{{get_value_at 'services' 'iam' 'credential_reports' id 'access_key_2_active'}}</span></div>
        <div class="list-group-item-text item-margin">Signing cert 1 active: {{get_value_at 'services' 'iam' 'credential_reports' id 'cert_1_active'}}</div>
        <div class="list-group-item-text item-margin">Signing cert 2 active: {{get_value_at 'services' 'iam' 'credential_reports' id 'cert_2_active'}}</div>
        <div class="list-group-item-text item-margin">Cert 1 last used: {{format_date (get_value_at 'services' 'iam' 'credential_reports' id 'access_key_1_last_used_date')}}</div>
        <div class="list-group-item-text item-margin">Cert 2 last used: {{format_date (get_value_at 'services' 'iam' 'credential_reports' id 'access_key_2_last_used_date')}}</div>
    </div>
  </script>

  <script>
    Handlebars.registerPartial("services.iam.credential_reports", $("#services\\.iam\\.credential_reports\\.partial").html());
  </script>

<!-- Single IAM credential_reports template -->
<script id="services.iam.credential_reports.template" type="text/x-handlebars-template">
    {{> modal-template template='services.iam.credential_reports'}}
</script>

<script>
    var single_iam_credential_reports_template = Handlebars.compile($("#services\\.iam\\.credential_reports\\.template").html());
</script>
