<!-- Redis Cache partial -->
<script id="services.rediscache.caches.partial" type="text/x-handlebars-template">
    <div id="resource-name" class="list-group-item active">
        <h4 class="list-group-item-heading">{{name}}</h4>
    </div>

    <div class="list-group-item">
        <h4 class="list-group-item-heading">Information</h4>
        <div class="list-group-item-text item-margin">Public access: <span id="rediscache.caches.{{@key}}.public_access_allowed">{{ convert_bool_to_enabled public_access_allowed }}</span></div>
    </div>

</script>

<script>
    Handlebars.registerPartial("services.rediscache.caches", $("#services\\.rediscache\\.caches\\.partial").html());
</script>

<!-- Single Redis Cache template -->
<script id="single_rediscache_caches-template" type="text/x-handlebars-template">
    {{> modal-template template='services.rediscache.caches' }}
</script>
<script>
    var single_rediscache_caches_template = Handlebars.compile($("#single_rediscache_caches-template").html());
</script>