
    <!-- Default accordion item -->
    <script id="services.vpc.regions.vpcs.peering_connections.partial" type="text/x-handlebars-template">
        <a href="javascript:showObject('services.vpc.regions.{{region}}.peering_connections.{{item}}')">
            {{get_value_at (concat 'services.vpc.regions' region 'peering_connections' item 'name')}}
            ({{get_value_at (concat 'services.vpc.regions' region 'peering_connections' item 'Status.Message')}},
             {{get_value_at (concat 'services.vpc.regions' region 'peering_connections' item 'peer_info.name')}})
        </a>
    </script>
    <script>
        Handlebars.registerPartial("services.vpc.regions.vpcs.peering_connections", $("#services\\.vpc\\.regions\\.vpcs\\.peering_connections\\.partial").html());
    </script>
