
    <!-- ELBv2 LB partial -->
    <script id="services.elbv2.regions.id.vpcs.id.lbs.partial" type="text/x-handlebars-template">
        <div class="list-group-item active">
            <h4 class="list-group-item-heading">{{name}}</h4>
        </div>
        <div class="list-group-item">
            <h4>Network</h4>
            <ul>
                <li class="list-group-item-text">VPC: {{get_value_at 'services.elbv2.regions' region 'vpcs' vpc 'name'}} ({{vpc}})</li>
                <li class="list-group-item-text">DNS: {{DNSName}}</li>
                <li class="list-group-item-text">Scheme: {{Scheme}}</li>
                <li class="list-group-item-text">Type: {{Type}}</li>
                <li class="list-group-item-text">Availability zones:</li>
                <ul>
                    {{#each AvailabilityZones}}
                        <li class="list-group-item-text">{{ZoneName}} ({{SubnetId}})</li>
                    {{/each}}
                </ul>
            </ul>
        </div>
        <div class="list-group-item">
            <h4>Listeners</h4>
            <ul>
                {{#each listeners}}
                    <li class="list-group-item-text">
                        {{@key}} ({{Protocol}}{{#if SslPolicy}}, {{SslPolicy}}{{/if}})
                    </li>
                {{/each}}
            </ul>
        </div>
        <div class="list-group-item">
            <h4>Attributes</h4>
            <ul>
                {{#each attributes}}
                    <li class="list-group-item-text">{{Key}}: <span id="elbv2.regions.{{../region}}.vpcs.{{../vpc}}.lbs.{{@../key}}.attributes.{{@key}}">{{Value}}</span></li>
                {{/each}}
            </ul>
        </div>
        <div class="list-group-item" style="padding-bottom: 0 !important;">
            <div class="accordion">
                <h4 class="list-group-item-heading accordion-heading">Security groups
                  {{> count_badge count=security_groups.length target=(concat '#services.elbv2.regions' region 'vpcs' vpc 'lbs' @key 'security_groups')}}
                </h4>
                <div id="services.elbv2.regions.{{region}}.vpcs.{{vpc}}.lbs.{{@key}}.security_groups" class="accordion-body collapse">
                    <div class="accordion-inner">
                        <ul class="no-bullet">
                            {{#each security_groups}}
                                <li><a href="javascript:showObject('services.ec2.regions.{{../region}}.vpcs.{{../vpc}}.security_groups.{{GroupId}}')">{{GroupId}}</a></li>
                            {{/each}}
                        </ul>
                    </div>
                </div>
            </div>
        </div>

    </script>
    <script>
      Handlebars.registerPartial("services.elbv2.regions.id.vpcs.id.lbs", $("#services\\.elbv2\\.regions\\.id\\.vpcs\\.id\\.lbs\\.partial").html());
    </script>

