
<!-- Stackdriver Logging sinks partial -->
<script id="services.stackdriverlogging.sinks.partial" type="text/x-handlebars-template">
    <div id="resource-name" class="list-group-item active">
        <h4 class="list-group-item-heading">{{name}}</h4>
    </div>
    <div class="list-group-item">
        <h4 class="list-group-item-heading">Information</h4>
        <div class="list-group-item-text item-margin">Name: <span id="stackdriverlogging.sinks.{{@key}}.name">{{name}}</span></div>
        <div class="list-group-item-text item-margin">Filter: <span id="stackdriverlogging.sinks.{{@key}}.filter"><code>{{filter}}</code></span></div>
        <div class="list-group-item-text item-margin">Destination: <span id="stackdriverlogging.sinks.{{@key}}.destination"><code>{{destination}}</code></span></div>
    </div>
</script>

<script>
    Handlebars.registerPartial("services.stackdriverlogging.sinks", $("#services\\.stackdriverlogging\\.sinks\\.partial").html());
</script>

<!-- Single stackdriverlogging sink template -->
<script id="single_stackdriverlogging_sink-template" type="text/x-handlebars-template">
    {{> modal-template template='services.stackdriverlogging.sinks'}}
</script>
<script>
    var single_stackdriverlogging_sink_template = Handlebars.compile($("#single_stackdriverlogging_sink-template").html());
</script>
