# -*- coding: utf-8 -*-

__author__ = 'NCC Group'
__version__ = '4.2.post2'

AWSCONFIG  = 42
EXCEPTIONS = 4242
HTMLREPORT = 424242
AWSRULESET = 42424242

DEFAULT_REPORT_DIR = 'scoutsuite-report'
AWSCONFIG_FILE = 'scoutsuite-results/scoutsuite_results.js'
AWSRULESET_FILE = 'scoutsuite-results/scoutsuite_ruleset.js'
EXCEPTIONS_FILE = 'scoutsuite-results/scoutsuite_exceptions.js'
HTMLREPORT_FILE = 'report.html'
GENERATOR_FILE = 'ruleset-generator.html'

REPORT_TITLE = 'Scout Suite Report'
