
    <!-- VPC partial -->
    <script id="services.vpc.regions.id.vpcs.partial" type="text/x-handlebars-template">
        <div id="resource-name" class="list-group-item active">
          <h4 class="list-group-item-heading">{{name}}</h4>
        </div>
        <div class="list-group-item">
          <h4>Information</h4>
          <ul>
            <li class="list-group-item-text">Region: {{region}}</li>
            <li class="list-group-item-text">ID: {{@key}}</li>
            <li class="list-group-item-text">name: {{name}}</li>
          </ul>
        </div>
        <div class="list-group-item" style="padding-bottom: 0 !important;">
          <div class="accordion">
            <h4 class="list-group-item-heading accordion-heading">Network ACLs
              {{> count_badge count=(count_vpc_network_acls network_acls) target=(concat '#services.vpc.regions' region 'vpcs' @key 'network_acls')}}
            </h4>
            <div id="services.vpc.regions.{{region}}.vpcs.{{@key}}.network_acls" class="accordion-body collapse">
              <div class="accordion-inner">
                <ul class="no-bullet">
                  {{#each network_acls}}
                    <li><a href="javascript:showObject('services.vpc.regions.{{../region}}.vpcs.{{@../key}}.network_acls.{{@key}}')">{{@key}}</a></li>
                  {{/each}}
                </ul>
              </div>
            </div>
          </div>
        </div>
        <div class="list-group-item" style="padding-bottom: 0 !important;">
          <div class="accordion">
            <h4 class="list-group-item-heading accordion-heading">Instances
                {{> count_badge count=(count_vpc_instances instances) target=(concat '#services.vpc.regions' region 'vpcs' @key 'instances')}}
            </h4>
            <div id="services.vpc.regions.{{region}}.vpcs.{{@key}}.instances" class="accordion-body collapse">
              <div class="accordion-inner">
                <ul class="no-bullet">
                  {{#each instances}}
                    <li><a href="javascript:showObject('services.vpc.regions.{{../region}}.vpcs.{{@../key}}.instances.{{@key}}')">{{@key}}</a></li>
                  {{/each}}
                </ul>
              </div>
            </div>
          </div>
        </div>
        <div class="list-group-item" style="padding-bottom: 0 !important">
            {{> accordion region = region items = peering_connections
                title = 'Peering connections'
                item_partial = 'services.vpc.regions.vpcs.peering_connections'
                accordion_id = (concat 'services.vpc.regions' region 'vpcs' @key 'peering_connections')
            }}
        </div>
    </script>
    <script>
      Handlebars.registerPartial("services.vpc.regions.id.vpcs", $("#services\\.vpc\\.regions\\.id\\.vpcs\\.partial").html());
    </script>

    <!-- Single VPC template -->
    <script id="single_vpc-template" type="text/x-handlebars-template">
        {{> modal-template template='services.vpc.regions.id.vpcs'}}
    </script>
    <script>
        var single_vpc_template = Handlebars.compile($("#single_vpc-template").html());
    </script>

