
  <!-- Generic partial for resource -->
  <script id="details.partial" type="text/x-handlebars-template">
    <div id="services.{{service_name}}.{{resource_type}}.details">
      {{#each items}}
        <div class="list-group" id="services.{{../service_name}}.{{../resource_type}}.{{@key}}.view">
          {{> (lookup .. 'partial_name') service_name = ../service_name resource_type = ../resource_type resource_key = @key resource = this}}
        </div>
      {{/each}}
    </div>
  </script>
  <script>
    Handlebars.registerPartial("details", $("#details\\.partial").html());
  </script>

